/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*- */

/*
 * Nautilus
 *
 * Copyright (C) 2010 ammonkey
 *
 * Nautilus is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * Nautilus is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; see the file COPYING.  If not,
 * write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: ammonkey <am.monkeyd@gmail.com>
 */

#include <config.h>
#include "nautilus-filter-bar.h"

#include <eel/eel-gtk-macros.h>
#include <gdk/gdkkeysyms.h>
#include <gtk/gtk.h>
#include <string.h>

#define NAUTILUS_FILTER_BAR_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), NAUTILUS_TYPE_FILTER_BAR, NautilusFilterBarPrivate))

struct NautilusFilterBarPrivate
{
        //GList           *btns;
        gint            zlist_idx;
        GtkWidget       *zoom_label;
};

enum {
	/*ACTIVATE,
	CANCEL,
	LOCATION_CHANGED,*/
        TIME_PERIOD_CHANGED,
	LAST_SIGNAL
};
static guint signals[LAST_SIGNAL];

G_DEFINE_TYPE (NautilusFilterBar, nautilus_filter_bar, GTK_TYPE_HBOX)

static void nautilus_filter_bar_class_init (NautilusFilterBarClass *class);
static void nautilus_filter_bar_init       (NautilusFilterBar      *bar);

//EEL_CLASS_BOILERPLATE (NautilusFilterBar, nautilus_filter_bar, GTK_TYPE_HBOX)

/*EEL_IMPLEMENT_MUST_OVERRIDE_SIGNAL (nautilus_navigation_bar, get_location)
EEL_IMPLEMENT_MUST_OVERRIDE_SIGNAL (nautilus_navigation_bar, set_location)*/

static void
nautilus_filter_bar_finalize (GObject *object)
{
	NautilusFilterBar *bar = NAUTILUS_FILTER_BAR (object);

	/* g_object_unref */
        
        //g_list_free (bar->priv->btns);  
        G_OBJECT_CLASS (nautilus_filter_bar_parent_class)->finalize (object);
}

static void
nautilus_filter_bar_class_init (NautilusFilterBarClass *klass)
{
	GObjectClass *object_class = G_OBJECT_CLASS (klass);
	//GtkBindingSet *binding_set;

       	object_class->finalize = nautilus_filter_bar_finalize;

        g_type_class_add_private (klass, sizeof (NautilusFilterBarPrivate));
	
	/*signals[ACTIVATE] = g_signal_new
		("activate",
		 G_TYPE_FROM_CLASS (object_class),
		 G_SIGNAL_RUN_LAST,
		 G_STRUCT_OFFSET (NautilusNavigationBarClass,
				    activate),
		 NULL, NULL,
		 g_cclosure_marshal_VOID__VOID,
		 G_TYPE_NONE, 0);

	signals[CANCEL] = g_signal_new
		("cancel",
		 G_TYPE_FROM_CLASS (object_class),
		 G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
		 G_STRUCT_OFFSET (NautilusNavigationBarClass,
				    cancel),
		 NULL, NULL,
		 g_cclosure_marshal_VOID__VOID,
		 G_TYPE_NONE, 0);

	signals[LOCATION_CHANGED] = g_signal_new
		("location_changed",
		 G_TYPE_FROM_CLASS (object_class),
		 G_SIGNAL_RUN_LAST,
		 G_STRUCT_OFFSET (NautilusNavigationBarClass,
				    location_changed),
		 NULL, NULL,
		 g_cclosure_marshal_VOID__STRING,
		 G_TYPE_NONE, 1, G_TYPE_STRING);*/

	/*klass->activate = NULL;
	klass->cancel = NULL;*/

	/*binding_set = gtk_binding_set_by_class (klass);
	gtk_binding_entry_add_signal (binding_set, GDK_Escape, 0, "cancel", 0);*/

	/*EEL_ASSIGN_MUST_OVERRIDE_SIGNAL (klass, nautilus_navigation_bar, get_location);
	EEL_ASSIGN_MUST_OVERRIDE_SIGNAL (klass, nautilus_navigation_bar, set_location);*/
	
        signals[TIME_PERIOD_CHANGED] = g_signal_new
		("time_period_changed",
		 G_TYPE_FROM_CLASS (object_class),
		 G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
		 G_STRUCT_OFFSET (NautilusFilterBarClass,
				    time_period_changed),
		 NULL, NULL,
		 //g_cclosure_marshal_VOID__STRING,
		 g_cclosure_marshal_VOID__INT,
		 G_TYPE_NONE, 1, G_TYPE_INT);
}

typedef struct
{
        gchar *label;
} InterpretationFilter;

static const InterpretationFilter if_list[] =
{
        { "All" },
        { "Audio" },
        { "Documents" },
        { "Images" },
        { "Developement" },
        { "Videos" },
        { "People" },
        { NULL }
};

typedef struct
{
        gchar   *label;
        gint    value;
} ZoomList;

static const ZoomList zlist[] =
{
        { "Today", 1 },
        { "This week", 7 },
        { "Last 2 weeks", 14 },
        { "This month", 30 },
        { "Last 2 months", 60 },
        { "Last 3 months", 90 },
        { "Last 6 months", 180},
        { NULL, 0}
};

/*
static void
button_clicked_callback (GtkWidget *button, NautilusFilterBar *bar)
{
        GList *l = bar->priv->btns;
        if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON (button)))
        {
                for (; l != NULL; l=l->next) {
                        if (l->data != button)
                                 gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (l->data), FALSE);
                }
        }
}*/

static void
time_zoom_in_button_clicked_callback (GtkWidget *button, NautilusFilterBar *bar)
{
        bar->priv->zlist_idx = (bar->priv->zlist_idx +1) % 7;
        gtk_label_set_text (GTK_LABEL (bar->priv->zoom_label), zlist[bar->priv->zlist_idx].label);
        g_signal_emit (bar, signals[TIME_PERIOD_CHANGED], 0, zlist[bar->priv->zlist_idx].value);
}

static void
time_zoom_out_button_clicked_callback (GtkWidget *button, NautilusFilterBar *bar)
{
        gint i;

        if ((i = bar->priv->zlist_idx -1) < 0)
                bar->priv->zlist_idx = 6;
        else
                bar->priv->zlist_idx = i;
        gtk_label_set_text (GTK_LABEL (bar->priv->zoom_label), zlist[bar->priv->zlist_idx].label);
        g_signal_emit (bar, signals[TIME_PERIOD_CHANGED], 0, zlist[bar->priv->zlist_idx].value);
}

#if 0
static void
interpretation_filter_button_new (NautilusFilterBar *bar, gint i)
{
        GtkWidget *hbox = GTK_WIDGET (bar);
        GtkWidget *btn;
        
        btn = gtk_toggle_button_new_with_label(if_list[i].label);
        gtk_button_set_relief (GTK_BUTTON (btn), GTK_RELIEF_NONE);
        gtk_box_pack_start (GTK_BOX (hbox), btn, FALSE, FALSE, 0);
	gtk_widget_show (btn);
        bar->priv->btns = g_list_append(bar->priv->btns, btn);

        /*g_signal_connect (btn,
                          "toggled",
			  G_CALLBACK (button_clicked_callback),
			  bar);*/
}
#endif

#if 0
void
nautilus_filter_bar_button_append (GtkWidget *hbox, gchar *label, gchar *interpretation)
{
        GtkWidget *btn;
        NautilusFilterBar *bar = NAUTILUS_FILTER_BAR (hbox);
        
        btn = gtk_toggle_button_new_with_label(label);
        gtk_button_set_relief (GTK_BUTTON (btn), GTK_RELIEF_NONE);
        gtk_box_pack_start (GTK_BOX (hbox), btn, FALSE, FALSE, 0);
	gtk_widget_show (btn);
        bar->priv->btns = g_list_append(bar->priv->btns, btn);
        
        /*g_signal_connect (btn,
                          "toggled",
			  G_CALLBACK (button_clicked_callback),
			  bar);*/
}
#endif

static void
nautilus_filter_bar_init (NautilusFilterBar *bar)
{
        gint i;
        GtkWidget *zin_btn;
        GtkWidget *zout_btn;
        GtkWidget *zoom_label;

        bar->priv = NAUTILUS_FILTER_BAR_GET_PRIVATE (bar);
        //bar->priv->btns = NULL;
        bar->priv->zlist_idx = 1;

        /* interration filters */
        /*for (i=0; if_list[i].label != NULL; i++)
                interpretation_filter_button_new (bar, i);*/
        /* zoom list widget */
        GtkWidget *l_zin = gtk_label_new(NULL);
        gtk_label_set_markup (GTK_LABEL (l_zin), "<span size='large'><b>+</b></span>");
        zin_btn = gtk_button_new();
        gtk_container_add (GTK_CONTAINER (zin_btn), l_zin);
        gtk_button_set_relief (GTK_BUTTON (zin_btn), GTK_RELIEF_NONE);
        gtk_box_pack_end (GTK_BOX (bar), zin_btn, FALSE, FALSE, 0);
	gtk_widget_show (l_zin);
	gtk_widget_show (zin_btn);
        g_signal_connect (zin_btn, "clicked", G_CALLBACK (time_zoom_in_button_clicked_callback), bar);

        GtkWidget *l_zout = gtk_label_new(NULL);
        gtk_label_set_markup (GTK_LABEL (l_zout), "<span size='large'><b>-</b></span>");
        zout_btn = gtk_button_new();
        gtk_container_add (GTK_CONTAINER (zout_btn), l_zout);
        gtk_button_set_relief (GTK_BUTTON (zout_btn), GTK_RELIEF_NONE);
        gtk_box_pack_end (GTK_BOX (bar), zout_btn, FALSE, FALSE, 0);
	gtk_widget_show (l_zout);
	gtk_widget_show (zout_btn);
        g_signal_connect (zout_btn, "clicked", G_CALLBACK (time_zoom_out_button_clicked_callback), bar);

        zoom_label = gtk_label_new(zlist[bar->priv->zlist_idx].label);
        gtk_box_pack_end (GTK_BOX (bar), zoom_label, FALSE, FALSE, 0);
	gtk_widget_show (zoom_label);
        bar->priv->zoom_label = zoom_label;
}

GtkWidget *
nautilus_filter_bar_new ()
{
        GObject *bar;

	bar = g_object_new (NAUTILUS_TYPE_FILTER_BAR, NULL);

	return GTK_WIDGET (bar);
}

gint
nautilus_filter_bar_get_period (NautilusFilterBar *bar)
{
        return (zlist[bar->priv->zlist_idx].value);
}

void
nautilus_filter_bar_set_period (NautilusFilterBar *bar, gint days)
{
        gint i=0;
        for (i=0; zlist[i].label != NULL; i++)
        {
                if (zlist[i].value == days)
                        break;
        }
        bar->priv->zlist_idx = i;
        gtk_label_set_text (GTK_LABEL (bar->priv->zoom_label), zlist[i].label);
}

/*
void
nautilus_navigation_bar_set_location (NautilusNavigationBar *bar,
				      const char *location)
{
	g_return_if_fail (NAUTILUS_IS_NAVIGATION_BAR (bar));

	EEL_CALL_METHOD (NAUTILUS_NAVIGATION_BAR_CLASS, bar,
			      set_location, (bar, location));
}

void
nautilus_navigation_bar_location_changed (NautilusNavigationBar *bar)
{
	char *location;

	g_return_if_fail (NAUTILUS_IS_NAVIGATION_BAR (bar));

	location = nautilus_navigation_bar_get_location (bar);
	g_signal_emit (bar,
			 signals[LOCATION_CHANGED], 0,
			 location);
	g_free (location);
}*/
